//+------------------------------------------------------------------+
//|                                            Check_FreezeLevel.mq5 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property description "  SYMBOL_TRADE_FREEZE_LEVEL  "
//---       _ 
#define Ask    SymbolInfoDouble(_Symbol,SYMBOL_ASK)
#define Bid    SymbolInfoDouble(_Symbol,SYMBOL_BID)
//+------------------------------------------------------------------+
//|                   |
//+------------------------------------------------------------------+
bool CheckOrderForFREEZE_LEVEL(ulong ticket)
  {
//---   SYMBOL_TRADE_FREEZE_LEVEL
   int freeze_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL);
   if(freeze_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_FREEZE_LEVEL=%d:   "+
                  "   %d    ",freeze_level,freeze_level);
     }
//---    
   if(!OrderSelect(ticket))
     {
      //---    
      return(false);
     }
//---   
   double price=OrderGetDouble(ORDER_PRICE_OPEN);
   double sl=OrderGetDouble(ORDER_SL);
   double tp=OrderGetDouble(ORDER_TP);
   ENUM_ORDER_TYPE type=(ENUM_ORDER_TYPE)OrderGetInteger(ORDER_TYPE);
//---   
   bool check=false;
//---   
   switch(type)
     {
      //---   BuyLimit
      case  ORDER_TYPE_BUY_LIMIT:
        {
         //---        
         check=((Ask-price)>freeze_level*_Point);
         if(!check)
            PrintFormat(" %s #%d  : Ask-Open=%d  < SYMBOL_TRADE_FREEZE_LEVEL=%d ",
                        EnumToString(type),ticket,(int)((Ask-price)/_Point),freeze_level);
         return(check);
        }
      //---   BuyLimit
      case  ORDER_TYPE_SELL_LIMIT:
        {
         //---        
         check=((price-Bid)>freeze_level*_Point);
         if(!check)
            PrintFormat(" %s #%d  : Open-Bid=%d  < SYMBOL_TRADE_FREEZE_LEVEL=%d ",
                        EnumToString(type),ticket,(int)((price-Bid)/_Point),freeze_level);
         return(check);
        }
      break;
      //---   BuyStop
      case  ORDER_TYPE_BUY_STOP:
        {
         //---        
         check=((price-Ask)>freeze_level*_Point);
         if(!check)
            PrintFormat(" %s #%d  : Ask-Open=%d  < SYMBOL_TRADE_FREEZE_LEVEL=%d ",
                        EnumToString(type),ticket,(int)((price-Ask)/_Point),freeze_level);
         return(check);
        }
      //---   SellStop
      case  ORDER_TYPE_SELL_STOP:
        {
         //---        
         check=((Bid-price)>freeze_level*_Point);
         if(!check)
            PrintFormat(" %s #%d  : Bid-Open=%d  < SYMBOL_TRADE_FREEZE_LEVEL=%d ",
                        EnumToString(type),ticket,(int)((Bid-price)/_Point),freeze_level);
         return(check);
        }
      break;
     }
//---    
   return (false);
  }
//+------------------------------------------------------------------+
//|  TP  SL                     |
//+------------------------------------------------------------------+
bool CheckPositionForFREEZE_LEVEL(ulong ticket)
  {
//---   SYMBOL_TRADE_FREEZE_LEVEL
   int freeze_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL);
   if(freeze_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_FREEZE_LEVEL=%d:   "+
                  "   %d    ",freeze_level,freeze_level);
     }
//---    
   if(!PositionSelectByTicket(ticket))
     {
      //---    
      return(false);
     }
//---        
   ENUM_POSITION_TYPE pos_type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
   double sl=PositionGetDouble(POSITION_SL);
   double tp=PositionGetDouble(POSITION_TP);
//---   StopLoss  TakeProfit
   bool SL_check=false,TP_check=false;
//---  
   switch(pos_type)
     {
      //--- 
      case POSITION_TYPE_BUY:
        {
         SL_check=(Bid-sl>freeze_level*_Point);
         if(!SL_check)
            PrintFormat(" %s #%d  : Bid-StopLoss=%d "+
                        " < SYMBOL_TRADE_FREEZE_LEVEL=%d )",
                        EnumToString(pos_type),ticket,(int)((Bid-sl)/_Point),freeze_level);
         TP_check=(tp-Bid>freeze_level*_Point);
         if(!TP_check)
            PrintFormat(" %s #%d  : TakeProfit-Bid=%d "+
                        " < SYMBOL_TRADE_FREEZE_LEVEL=%d )",
                        EnumToString(pos_type),ticket,(int)((tp-Bid)/_Point),freeze_level);
         //---   
         return(SL_check&&TP_check);
        }
      break;
      //--- 
      case POSITION_TYPE_SELL:
        {
         SL_check=(sl-Ask>freeze_level*_Point);
         if(!SL_check)
            PrintFormat(" %s  : StopLoss-Ask=%d "+
                        " < SYMBOL_TRADE_FREEZE_LEVEL=%d )",
                        EnumToString(pos_type),(int)((sl-Ask)/_Point),freeze_level);
         TP_check=(Ask-tp>freeze_level*_Point);
         PrintFormat(" %s  : Ask-TakeProfit=%d "+
                     " < SYMBOL_TRADE_FREEZE_LEVEL=%d )",
                     EnumToString(pos_type),(int)((Ask-tp)/_Point),freeze_level);
         //---   
         return(SL_check&&TP_check);
        }
      break;
     }
//---    
   return (false);
  }
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//---    
   ulong order_ticket,pos_ticket;
   ENUM_POSITION_TYPE position_type;
/*
   ...    
*/
//---      SYMBOL_TRADE_FREEZE_LEVEL
   if(CheckOrderForFREEZE_LEVEL(order_ticket))
     {
      //   ,  
     }
/*
   ...    
*/
//---     
   if(CheckPositionForFREEZE_LEVEL(pos_ticket))
     {
      //---   ,        
     }
  }
//+------------------------------------------------------------------+
