//+------------------------------------------------------------------+
//|                               Test_SYMBOL_TRADE_FREEZE_LEVEL.mq4 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "http://forum.mql4.com/ru/69818/page2#1023827"
#property version   "1.00"
#property description "      SYMBOL_TRADE_FREEZE_LEVEL"
#property strict
//---  SYMBOL_TRADE_STOPS_LEVEL  SYMBOL_TRADE_FREEZE_LEVEL
int freeze_level,stops_level;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---    ,        
   freeze_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL);
   if(freeze_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_FREEZE_LEVEL=%d:     ,"+
                  "   %d    ",freeze_level,freeze_level);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
double GetActivationPrice(int ticket)
  {
//---
   double activation_price=0;
//---
   if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
     {
      int type=OrderType();
      //---    Ask
      if(type==OP_BUYLIMIT || type==OP_BUYSTOP)
        {
         activation_price=Ask;
        }
      else
      //---    Bid
      if(type==OP_SELLLIMIT || type==OP_SELLSTOP)
        {
         activation_price=Bid;
        }
     }
//---      
   return activation_price;
  }
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+
double GetNearestPrice(int type,int distance)
  {
   double price=0;
//---   
   switch(type)
     {
      case  OP_SELLSTOP:
         price=Bid-distance*_Point;
         break;
      case  OP_BUYLIMIT:
         price=Ask-distance*_Point;
         break;
      case  OP_SELLLIMIT:
         price=Bid+distance*_Point;
         break;
      case  OP_BUYSTOP:
         price=Ask+distance*_Point;
         break;
      default:
         break;
     }
//---   
   return(NormalizeDouble(price,_Digits));
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   int orders=OrdersTotalPending(_Symbol);
//---       
   int distance,ticket;
//---  
   double activation_price;
//---         
   if(orders==1)
     {
      //---    
      if(OrderPendingSelect(0,_Symbol))
        {
         ticket=OrderTicket();
         //---  
         activation_price=GetActivationPrice(ticket);
         //---         
         distance=(int) MathRound(MathAbs(activation_price-OrderOpenPrice())/_Point);
         //---         
         freeze_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL);
         //---          
         stops_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
         //---   -      
         int min=MathMax(freeze_level,stops_level);
         double nearest_price=GetNearestPrice(OrderType(),min+1);
         //---    
         double openprice=OrderOpenPrice();
         //---   StopLoss  TakeProfit
         double sl=OrderStopLoss();
         double tp=OrderTakeProfit();
         datetime expiration=(datetime)OrderExpiration();

         //---     
         if(distance<freeze_level)
           {
            PlaySound("stops.wav"); //   
            Comment(StringFormat("Distance=%d [SYMBOL_TRADE_FREEZE_LEVEL=%d]  %s  CurrentPrice=%.5f",
                    distance,freeze_level,GetOrderTypeString(OrderType()),activation_price));
            //---        ,   
            if(MathAbs(nearest_price-openprice)>_Point)
              {
               //---     
               PrintFormat("Try to modify %s #%d at %.5f ==> new price=%.5f:  Bid=%.5f  Ask=%.5f",
                           GetOrderTypeString(OrderType()),ticket,openprice,nearest_price,Bid,Ask);

               //---    
               if(!OrderModify(ticket,nearest_price,sl,tp,expiration))
                 {
                  PrintFormat("freeze_level=%d",freeze_level);
                  //---   
                  PrintFormat("OrderModify for %s #%d failed, Error=%d",
                              GetOrderTypeString(OrderType()),ticket,GetLastError());
                 }
               else
                  PrintFormat("Modification of order %s #%d done successfully",
                              GetOrderTypeString(OrderType()),ticket);
               Print(" -------------");
              }
            //---       -        
            return;
           }
         if(distance>2*min)
           {
            //---         
            nearest_price=GetNearestPrice(OrderType(),stops_level+1);
            //---        
            int current_delta=(int)MathAbs((openprice-activation_price)/_Point);
            //---       
            if(MathAbs(nearest_price-openprice)/_Point>=1)
              {
               //---   
               PrintFormat("Modify order %s #%d at %.5f => %.5f  OpenPrice-CurrenPrice=%.5f-%.5f=%d points Ask=%.5f Bid=%.5f",
                           GetOrderTypeString(OrderType()),ticket,openprice,nearest_price,openprice,activation_price,current_delta,Ask,Bid);
               //--- 
               if(OrderModify(ticket,nearest_price,sl,tp,expiration))
                 {
                  PrintFormat("Order %s #%d modified succesfully",GetOrderTypeString(OrderType()),ticket);
                 }
               //---   
               else
                 {
                  //---   
                  PrintFormat("OrderModify for %s #%d failed, Error=%d",
                              GetOrderTypeString(OrderType()),ticket,GetLastError());
                 }
              }
            //---        -        
            return;
           }
         //---     
        }
     }
//--- OnTick() 
  }
//+------------------------------------------------------------------+
//|               |
//+------------------------------------------------------------------+
int OrdersTotalPending(string sym)
  {
   int orders=0;
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
        {
         if(OrderSymbol()==sym)
           {
            if((OrderType()!=OP_BUY) && (OrderType()!=OP_SELL))
              {
               orders++;
              }
           }
        }
     }
   return(orders);
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
int OrderPendingSelect(int ind,string sym)
  {
   int ticket=0,counter=0;
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
        {
         if(OrderSymbol()==sym)
           {
            if((OrderType()!=OP_BUY) && (OrderType()!=OP_SELL))
              {
               if(counter==ind)
                 {
                  ticket=OrderTicket();
                  break;
                 }
               counter++;
              }
           }
        }
     }

   return(ticket);
  }
//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
string GetOrderTypeString(int type)
  {
   switch(type)
     {
      case  OP_BUY:
         return("OP_BUY");       break;
      case  OP_SELL:
         return("OP_SELL");      break;
      case  OP_BUYLIMIT:
         return("OP_BUYLIMIT");  break;
      case  OP_SELLLIMIT:
         return("OP_SELLLIMIT"); break;
      case  OP_BUYSTOP:
         return("OP_BUYSTOP");   break;
      case  OP_SELLSTOP:
         return("OP_SELLSTOP");  break;
     }
//---        
   return("Unknown order type");
  }
//+------------------------------------------------------------------+
